# -*- coding:utf-8 -*-
import itertools as it
import sys
import argparse
from zca_snippets.detector import InfoDetector, positional
import json #xxx:

class ZCASnippets(object):
    def __init__(self, io, detector, verbose=True, base_depends=True):
        self.io = io
        self.detector = detector
        self.verbose = verbose
        self.base_depends = base_depends

    def out(self, s):
        self.io.write(s)
        self.io.write("\n")

    def dump_snippets(self, iface, clsname):
        module = self.detector.module_path(iface)
        ifacename = self.detector.object_name(iface)
        if self.verbose:
            if self.base_depends:
                self.out("from zope.interface import implementer")
            self.out("from {module} import {name}".format(
                module=module,
                name=ifacename))
            self.out("")


        self.out("## see: {module}:{name}".format(module=module, name=ifacename))
        self.out("@implementer({name})".format(name=self.detector.object_name(iface)))
        self.out("class {clsname}(object):".format(clsname=self.detector.implements_name(iface, clsname)))
        for attr in self.detector.parse_attributes(iface):
            self.out("""\
    @property
    def {attr[name]}(self):
        pass
""".format(attr=attr))
        for method in self.detector.parse_methods(iface):
            required = (k for k,v in method["positional"] if v is positional)
            optionals = ("=".join((k, json.dumps(v))) for k, v in method["positional"] if not v is positional)
            args = ", ".join(it.chain(required, optionals))
            self.out("""\
    def {method[name]}(self, {args}{varargs}{kwargs}):
        pass
""".format(method=method, 
                       args=args, 
                       varargs="" if method["args"] is None else ", *{}".format(method["args"]), 
                       kwargs="" if method["kwargs"] is None else ", **{}".format(method["kwargs"])
                   ))



def main():
    parser = argparse.ArgumentParser(description="-")
    parser.add_argument("--quiet", "-q", action="store_true")
    parser.add_argument("--base", "-b", action="store_true")
    parser.add_argument("program")
    parser.add_argument("iface")
    parser.add_argument("name", nargs="?")
    parser.set_defaults(quiet=False, base=False)
    args = parser.parse_args(sys.argv)
    verbose = not args.quiet

    detector = InfoDetector()
    iface = detector.maybe_dotted(args.iface)
    snippets = ZCASnippets(sys.stdout, detector, verbose=verbose, base_depends=args.base)
    snippets.dump_snippets(iface, args.name)



# -*- coding:utf-8 -*-
import sys
import argparse
from zca_snippets.detector import InfoDetector
from zope.interface.interface import InterfaceClass

class ZCAListing(object):
    def __init__(self, io, detector):
        self.io = io
        self.detector = detector

    def dump_interfaces(self, module):
        for k, v in module.__dict__.items():
            if isinstance(v, InterfaceClass):
                module = self.detector.module_path(v)
                ifacename = self.detector.object_name(v)
                self.io.write("{}:{}\n".format(module, ifacename))

def main():
    parser = argparse.ArgumentParser(description="-")
    parser.add_argument("program")
    parser.add_argument("module")
    args = parser.parse_args(sys.argv)

    detector = InfoDetector()
    module = detector.maybe_dotted(args.module)
    listing = ZCAListing(sys.stdout, detector)
    listing.dump_interfaces(module)



from setuptools import setup, find_packages
requires = [
    "zope.interface"
    ]

setup(name='zca_snippets',
      version='0.0.2',
      description='snippets for zope.interface',
      long_description="", 
      author='podhmo',
      classifiers=[
          'Programming Language :: Python',
          'Programming Language :: Python :: 3'
      ],
      package_dir={'': '.'},
      packages=find_packages('.'),
      install_requires = requires,
      test_suite="zca_snippets.tests", 
      entry_points = """
      [console_scripts]
      zca-snippets = zca_snippets.snippets:main
      zca-list = zca_snippets.list:main
      """,
      )



# -*- coding:utf-8 -*-
import pkg_resources
from zope.interface.interface import Method

def import_symbol(symbol): #todo cache
    return pkg_resources.EntryPoint.parse("x=%s" % symbol).load(False)

class _positional(object):
    def __repr__(self):
        return "<_P>"
positional = _positional()

class InfoDetector(object):
    def maybe_dotted(self, s):
        return import_symbol(s)

    def module_path(self, obj):
        return obj.__module__

    def object_name(self, obj):
        return obj.__name__ 

    def implements_name(self, iface, name):
        if name:
            return name
        else:
            if iface.__name__.startswith("I"):
                return iface.__name__[1:]
            raise Exception("NoName")

    def parse_methods(self, iface):
        for name, desc in iface.namesAndDescriptions(True):
            if isinstance(desc, Method):
                args = [(k, desc._optional.get(k, positional)) for k in desc.positional]
                yield({"name": name, "positional":args, "args": desc.varargs, "kwargs": desc.kwargs, "doc":desc.__doc__})

    def parse_attributes(self, iface):
        for name, desc in iface.namesAndDescriptions(True):
            if not isinstance(desc, Method):
                yield({"name": name, "doc": desc.__doc__})

